%% Pterofin Watertunnel Test - ProcessMeanFile
%
% Author: Justin M. Walsh
%
% Created: 21 June 2023
%
% Why: New and necessary
%
% Date of last revision:
%
% Revision History:

%%%%%% THIS SCRIPT RECALCULATES TORQUE FROM VOLTAGES,
%%%%%% AS THEY WERE INCORRECTLY CALCULATED FROM THE SETUP FILE PRIOR TO
%%%%%% TEST 18

close all

%% user inputs and problem setup
MakePlots = 1; %1 if you want plots
SavePlots = 0; %1 if you want to save plots
SaveData = 0; %1 if you want to save data
SavePath = ''; %path for saving plots
ApplyTare = 1; % 0 = no tare, 1 = subtract mean torque tare
SweptArea = 0.0238*3.28^2; %Area swept by the fin (ft^2)
Dtip_ft = 2*11.875/12; % Tip diameter (ft)
ReC_min = 2e5; %Minimum chord-based Reynolds number

FC = 2/12; %fin mean chord (ft)

%% no user changes below this line

% prompt user to select .csv data file(s)

if exist('Data','var')
    LoadData = 0;
    disp('Using previously loaded data.')
elseif exist('FileName','var') && exist('PathName','var')
    LoadData = 1;
    disp('Loading previously selected data files.')
    FileNames = FileName;
else
    LoadData = 1;
    disp('Please select data files to load.')
    [FileName,PathName] = uigetfile('*.csv','Select the data file','MultiSelect','on');
    FileNames = FileName;
end

runnames = '';

if iscell(FileName)
    numFiles = length(FileName);
else
    numFiles = 1;
end

%% loop through all data files and store the data in structures
if LoadData
    for i=1:numFiles

        % load a data file
        if iscell(FileName)
            if iscell(PathName)
                loadfile=[PathName{i}, FileName{i}];
            else
                loadfile=[PathName, FileName{i}];
            end
            fname = FileName{i};
            str = ['Loading file: ' fname];
            disp(str)
        else
            loadfile=[PathName, FileName];
            fname = FileName;
            str = ['Loading file: ' fname];
            disp(str)
            clear FileName
            FileName{i} = fname;
            FileNames = FileName;
        end

        [runpath,runname,runext] = fileparts(fname);

        runnames = strcat(runnames, runname);

        [Data{i}, TimeVector{i}, Setup{i}]=read_a_csv(loadfile);

    end
end


%% loop through and priocess the data structures
for i=1:numFiles

    % Assign more readable names
    Time = Data{i}.Time; %(s)
    rho = Data{i}.Density;
    Vinf = Data{i}.Vinf;
    MotorRPM = Data{i}.EncBoxRPM;
    TestNo = str2num(FileName{i}(9:11));
    Test(i) = TestNo;

    % Pick tare coefficients based on when the data was taken
    if TestNo<5
        TorqueTareCoeffs = [2.2703E-12; - 7.0408E-09; + 7.9759E-06; - 4.3735E-03; - 8.6529E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin001 w/28t pitch pulley
    elseif TestNo<9
        TorqueTareCoeffs = [9.6731E-13; - 3.4633E-09; + 4.5051E-06; - 2.8778E-03; - 1.2160E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin006 w/23t pitch pulley
    elseif TestNo<19
        TorqueTareCoeffs = [2.1839E-12; - 7.1488E-09; + 7.8119E-06; - 3.8968E-03; - 9.9810E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin009 w/28t pitch pulley
    elseif TestNo<20
        TorqueTareCoeffs = [0; -9.7616E-10; + 1.8563E-06; - 1.5695E-03; - 8.2200E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin019 w/28t pitch pulley
    elseif TestNo==25
        TorqueTareCoeffs = [1.6696E-12; - 5.4962E-09; + 5.8498E-06; - 2.8427E-03; - 5.9340E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin027 w/18t pitch pulley
    elseif TestNo<29
        TorqueTareCoeffs = [1.3947E-12; - 4.5325E-09; + 4.8100E-06; - 2.3151E-03; - 5.9495E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin020 w/23t pitch pulley
    elseif TestNo<32
        TorqueTareCoeffs = [-1.2169E-13; - 2.4978E-10; + 6.2385E-07; - 8.5693E-04; - 9.2603E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin029 w/38t pitch pulley
    elseif TestNo<33
        TorqueTareCoeffs = [-4.9119E-13; + 1.2347E-09; - 8.8961E-07; - 4.3711E-04; - 1.7436E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin032 w/28t pitch pulley
    elseif TestNo<35
        TorqueTareCoeffs = [2.2361E-12; - 5.8101E-09; + 5.2869E-06; - 2.9465E-03; - 1.0790E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin033 w/28t pitch pulley
        FC = 1/12; %fin mean chord for Fin "E" (ft)
    elseif TestNo<36
        TorqueTareCoeffs = [2.2361E-12; - 5.8101E-09; + 5.2869E-06; - 2.9465E-03; - 1.0790E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin033 w/28t pitch pulley
        FC = 1.5/12; %fin mean chord for Fin "D"(ft)
    elseif TestNo<42
        TorqueTareCoeffs = [2.2361E-12; - 5.8101E-09; + 5.2869E-06; - 2.9465E-03; - 1.0790E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin033 w/28t pitch pulley
        FC = 2/12; %fin mean chord for Fin "C" (ft)
    else
        TorqueTareCoeffs = [-3.1454E-13; + 7.8328E-10; - 5.6519E-07; - 7.8943E-04; - 9.4018E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin042 w/28t pitch pulley
    end

    % apply mean torque tare
    if ApplyTare
        Tare_FtLbf = 0.7376*(TorqueTareCoeffs(1).*MotorRPM.^4 + ...
            TorqueTareCoeffs(2).*MotorRPM.^3 + ...
            TorqueTareCoeffs(3).*MotorRPM.^2 + ...
            TorqueTareCoeffs(4).*MotorRPM.^1 + ...
            TorqueTareCoeffs(5)); % Torque tare, ft-lbf
        disp('Applying mean torque tare.')
    else
        Tare_FtLbf = 0;
        disp('Torque tare not applied.')
    end

    % Calculate power coefficient
    Torque_Nm_temp = Data{i}.Torque_Signal*10.027798; %torque calculated from voltage without mechanism tare applied (Nm)
    Torque_Tared_FtLbf = Torque_Nm_temp*0.7376 - Tare_FtLbf; %torque calculated from voltage with mechanism tare applied (ft-lbf)
    clear Torque_Nm_temp
    Torque__Tared_Nm = Torque_Tared_FtLbf/0.7376; %torque calculated from voltage with mechanism tare applied (Nm)
    MotorPower_HP = MotorRPM.*Torque_Tared_FtLbf/5252; %Motor shaft power (hp)
    Cp = MotorPower_HP*550./(0.5*rho.*Vinf.^3*SweptArea);

    % Calculate torque coefficient
    KQ = Torque_Tared_FtLbf./(rho.*(MotorRPM/60).^2*(Dtip_ft)^5);
    
    % Calculate static chord Reynolds number (does not use blade relative velocity)
    ReC = FC*Vinf./Data{i}.Viscosity;

    % Calculate Reduced Frequency
    FinFreq = MotorRPM/60*pi; % rad/s, includes factor of 1/2 to account for 2:1 motor:fin speed ratio
    k = FC*FinFreq./Vinf; % reduced frequency of fin flapping and pitching motion

    % Identify peak Cp point for high Reynolds Number cases (not
    % considering low Re due to velocity measurement uncertainty in early
    % tests). Assign corresponding measurement values for later assembly
    % into a table for export.
    ind_hi_rel = find(ReC > ReC_min);
    Cp_max = max(Cp(ind_hi_rel));
    if ~isempty(Cp_max)
        ind_Cp_max = find(Cp == Cp_max);
        Run_Cp_max(i) = ind_Cp_max;
        k_Cp_max(i) = k(ind_Cp_max);
        Tare_FtLbf_Cp_max(i) = Tare_FtLbf(ind_Cp_max);
        Torque_Tared_Nm_Cp_max(i) = Torque__Tared_Nm(ind_Cp_max);
        Torque_Tared_FtLbf_Cp_max(i) = Torque_Tared_FtLbf(ind_Cp_max);
        MotorRPM_Cp_max(i) = MotorRPM(ind_Cp_max);
        KQ_Cp_max(i) = KQ(ind_Cp_max);
        MotorPower_HP_Cp_max(i) = MotorPower_HP(ind_Cp_max);
        Cp_Cp_max(i) = Cp(ind_Cp_max);
        ReC_Cp_max(i) = ReC(ind_Cp_max);
    else
        k_Cp_max(i) = nan;
        Run_Cp_max(i) = nan;
        Tare_FtLbf_Cp_max(i) = nan;
        Torque_Tared_Nm_Cp_max(i) = nan;
        Torque_Tared_FtLbf_Cp_max(i) = nan;
        MotorRPM_Cp_max(i) = nan;
        KQ_Cp_max(i) = nan;
        MotorPower_HP_Cp_max(i) = nan;
        Cp_Cp_max(i) = nan;
        ReC_Cp_max(i) = nan;
    end
    
    % Build a table of run data for export to CSV
    T = table(Time,k,Tare_FtLbf,Torque__Tared_Nm,Torque_Tared_FtLbf,MotorRPM,KQ,MotorPower_HP,Cp,ReC);

    % find indices of velocity > 3 ft/s (1 m/s) to filter out tare checks which are
    % performed with zero tunnel impeller RPM, but nonetheless may have
    % non-zero Vinf due to the small fluctations in the differential
    % pressure measurement.

    nonzero_ind = find(Vinf > 3);

    % generate figures
    figure;subplot(5,1,1);
    scatter(k(nonzero_ind),Torque__Tared_Nm(nonzero_ind),20,ReC(nonzero_ind))
    hold on;
    plot(k_Cp_max(i),Torque_Tared_Nm_Cp_max(i),'xr')
    ylabel('Motor Torque, Q (Nm)');
    xlim([0 1])
    c = colorbar;
    c.Label.String = 'Fin chord Reynolds Num.';
    grid on;

    subplot(5,1,2);
    scatter(k(nonzero_ind),MotorRPM(nonzero_ind),20,ReC(nonzero_ind))
    hold on;
    plot(k_Cp_max(i),MotorRPM_Cp_max(i),'xr')
    ylabel('Motor Speed (RPM)')
    xlim([0 1])
    c = colorbar;
    c.Label.String = 'Fin chord Reynolds Num.';
    grid on;

    subplot(5,1,3);
    scatter(k(nonzero_ind),KQ(nonzero_ind),20,ReC(nonzero_ind))
    hold on;
    plot(k_Cp_max(i),KQ_Cp_max(i),'xr')
    ylabel('Motor Torque Coef., {K_Q}')
    xlim([0 1])
    c = colorbar;
    c.Label.String = 'Fin chord Reynolds Num.';
    grid on;

    subplot(5,1,4);
    scatter(k(nonzero_ind),Cp(nonzero_ind),20,ReC(nonzero_ind))
    hold on;
    plot(k_Cp_max(i),Cp_Cp_max(i),'xr')
    ylabel('Motor Power Coef., {C_P}')
    xlim([0 1])
    c = colorbar;
    c.Label.String = 'Fin chord Reynolds Num.';
    grid on;

    subplot(5,1,5);
    scatter(k(nonzero_ind),Cp(nonzero_ind),20,ReC(nonzero_ind))
    hold on;
    plot(k_Cp_max(i),Cp_Cp_max(i),'xr')
    xlabel('Reduced Freq, k');
    ylabel('Motor Power Coef., {C_P}')
    xlim([0 1])
    ylim([0 0.1])
    c = colorbar;
    c.Label.String = 'Fin chord Reynolds Num.';
    grid on;

    titleString=['Time Averaged Values: ' FileName{i}(1:end-9)];
    sgtitle(titleString)
    set(gcf,'Position',[50,50,600,800])

    if SavePlots
        str = [SavePath FileName{i}(1:end-9) '_time_averaged_values.fig'];
        savefig(gcf,str)
        str = [SavePath FileName{i}(1:end-9) '_time_averaged_values.png'];
        exportgraphics(gcf,str)
    end

    if SaveData
        str = [PathName FileName{i}(1:end-9) '_MEAN_TARED.csv'];
        writetable(T,str)
    end
end

% Format and save data that spans all tests
TCP_var_names = {'Test','Run','k','Tare_FtLbf','Torque_Tared_Nm','Torque_Tared_FtLbf','MotorRPM','KQ','MotorPower_HP','Cp','ReC'};
TCP = table(Test',Run_Cp_max',k_Cp_max',Tare_FtLbf_Cp_max',Torque_Tared_Nm_Cp_max',...
    Torque_Tared_FtLbf_Cp_max',MotorRPM_Cp_max',KQ_Cp_max',MotorPower_HP_Cp_max',...
    Cp_Cp_max',ReC_Cp_max','VariableNames',TCP_var_names);
if SaveData
    str = ['Max_Cp_Table.csv'];
    writetable(TCP,str)
end

% Plot data that spans all tests (peak Cp values)
figure
plot(k_Cp_max,Cp_Cp_max,'ko')
text(k_Cp_max,Cp_Cp_max,num2cell(Test),'VerticalAlignment','bottom', ...
    'HorizontalAlignment','left')
xlabel('Reduced Freq, k');
ylabel('Motor Power Coef., {C_P}');
grid on;

if SavePlots
    str = [SavePath 'max_cp_values.fig'];
    savefig(gcf,str)
    str = [SavePath 'max_cp_values.png'];
    exportgraphics(gcf,str)
end

% Plot data that spans chord variation tests (peak Cp values)
figure
plot([1,1.5,2]/2,Cp_Cp_max(14:16),'ko')
text([1,1.5,2]/2,Cp_Cp_max(14:16),num2cell(Test(14:16)),'VerticalAlignment','bottom', ...
    'HorizontalAlignment','left')
xlabel('(Chord) / (max. chord tested)');
ylabel('Motor Power Coef., {C_P}');
grid on;

if SavePlots
    str = [SavePath 'max_cp_values_chord_variation.fig'];
    savefig(gcf,str)
    str = [SavePath 'max_cp_values_chord_variation.png'];
    exportgraphics(gcf,str)
end

% Plot data that spans pitch amplitude tests (peak Cp values)
figure
plot([1.5,2,2,2.5,2.5,3],Cp_Cp_max([13,8,12,9,10,11]),'ko')
text([1.5,2,2,2.5,2.5,3],Cp_Cp_max([13,8,12,9,10,11]),num2cell(Test([13,8,12,9,10,11])),'VerticalAlignment','bottom', ...
    'HorizontalAlignment','left')
xlabel('(Pitch amplitude) / (flap amplitude)');
ylabel('Motor Power Coef., {C_P}');
grid on;

if SavePlots
    str = [SavePath 'max_cp_values_pitch_variation.fig'];
    savefig(gcf,str)
    str = [SavePath 'max_cp_values_pitch_variation.png'];
    exportgraphics(gcf,str)
end