%% Pterofin Watertunnel Test - ProcessTimeSeriesFile
% Author: Justin M. Walsh
%
% Created: 21 June 2023
%
% Why: New and necessary
%
% Date of last revision:
%
% Revision History:

%%%%%% THIS SCRIPT RECALCULATES TORQUE FROM VOLTAGES,
%%%%%% AS THEY WERE INCORRECTLY CALCULATED FROM THE SETUP FILE PRIOR TO
%%%%%% TEST 18

close all
clc
%% user inputs and problem setup
MakePlots = 1; %1 if you want plots
SavePlots = 0; %1 if you want to save plots
SaveData = 0; %1 if you want to save data. Data will be saved to the LoadData path.
SavePath = ''; %path for saving plots
ApplyTare = 1; % 0 = no tare, 1 = subtract mean torque tare
SweptArea = 0.0238*3.28^2; %Area swept by the fin (ft^2)
Rtip =  0.9896; % Fin tip radius (ft)
FC = 2/12; %fin mean chord (ft)
Dtip_ft = Rtip*2; % Tip diameter (ft)

%% Maximum Cp cases for creating report figures (adjust based on user data path)
PathName = {'C:\2023\06\22\Pterofin0004\',...
'C:\2023\06\23\Pterofin0008\',...
'C:\2023\06\28\Pterofin0012\',...
'C:\2023\06\29\Pterofin0014\',...
'C:\2023\07\06\Pterofin0024\',...
'C:\12-inch WT test\2023\07\06\Pterofin0025\',...
'C:\12-inch WT test\2023\07\10\Pterofin0030\',...
'C:\12-inch WT test\2023\07\12\Pterofin0034\',...
'C:\12-inch WT test\2023\07\12\Pterofin0035\',...
'C:\12-inch WT test\2023\07\13\Pterofin0041\'};

FileName = {'Pterofin004R043.csv',...
'Pterofin008R033.csv',...
'Pterofin012R045.csv',...
'Pterofin014R045.csv',...
'Pterofin024R046.csv',...
'Pterofin025R029.csv',...
'Pterofin030R023.csv',...
'Pterofin034R059.csv',...
'Pterofin035R045.csv',...
'Pterofin041R036.csv'};

%% no user changes below this line

% prompt user to select .csv data file(s)

if exist('Data','var')
    LoadData = 0;
    disp('Using previously loaded data.')
elseif exist('FileName','var') && exist('PathName','var')
    LoadData = 1;
    disp('Loading previously selected data files.')
    FileNames = FileName;
else
    LoadData = 1;
    disp('Please select data files to load.')
    [FileName,PathName] = uigetfile('*.csv','Select the data file','MultiSelect','on');
    FileNames = FileName;
end

runnames = '';

if iscell(FileName)
    numFiles = length(FileName);
else
    numFiles = 1;
end

%% loop through all data files and store data in structures
if LoadData
    for i=1:numFiles
        
        % load a data file
        if iscell(FileName)
            if iscell(PathName)
                loadfile=[PathName{i}, FileName{i}];
            else
                loadfile=[PathName, FileName{i}];
            end
            fname = FileName{i};
            str = ['Loading file: ' fname];
            disp(str)
        else
            loadfile=[PathName, FileName];
            fname = FileName;
            str = ['Loading file: ' fname];
            disp(str)
            clear FileName
            FileName{i} = fname;
            FileNames = FileName;
        end
        
        [runpath,runname,runext] = fileparts(fname);
        
        runnames = strcat(runnames, runname);
        
        [Data{i}, TimeVector{i}, Setup{i}]=read_a_csv(loadfile);
        
    end
end


%% loop through all data structures
for i=1:numFiles   
    
    % Assign more readable names
    Time = Data{i}.Time; %(s)
    rho = mean(Data{i}.Density); %mean density to be used in phase average calculations
    Viscosity = mean(Data{i}.Viscosity); %mean viscosity to be used in phase average calculations
    Vinf = Data{i}.Vinf;
    MotorRPM = Data{i}.EncBoxRPM;
    TestNo = str2num(FileName{i}(9:11));
    MotorHomeEnc = Data{i}.raw_HomePulse;
    MotorPosition = Data{i}.Position;
    
    fourierFit=fit(MotorPosition,MotorHomeEnc,'fourier8');
    fourierFittedData=fourierFit(MotorPosition);
    [fourierPeaks,fourierLocs]=findpeaks(fourierFittedData,'MinPeakHeight',0.75*max(fourierFittedData));
    edgeLocs=fourierLocs;
    edgePeaks=fourierPeaks;
    
    % 2 rotations of the motor (every second edgeLoc) is needed to capture a
    % full back and forth motion of the hydrofoil. Need to capture a full
    % back and forth motion since flapping motion is not perfectly symmetric
    locsFullFlappingPeriods=[];
    numFullFlappingPeriods=floor(length(edgeLocs)/2);
    for I=1:numFullFlappingPeriods
        locsFullFlappingPeriods(I)=edgeLocs(2*I-1);
        peaksFullFlappingPeriods(I)=edgePeaks(2*I-1);
    end
    
    % Pick tare coefficients based on when the data was taken
    if TestNo<5
        TorqueTareCoeffs = [2.2703E-12; - 7.0408E-09; + 7.9759E-06; - 4.3735E-03; - 8.6529E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin001 w/28t pitch pulley
    elseif TestNo<9
        TorqueTareCoeffs = [9.6731E-13; - 3.4633E-09; + 4.5051E-06; - 2.8778E-03; - 1.2160E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin006 w/23t pitch pulley
    elseif TestNo<19
        TorqueTareCoeffs = [2.1839E-12; - 7.1488E-09; + 7.8119E-06; - 3.8968E-03; - 9.9810E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin009 w/28t pitch pulley
    elseif TestNo<20
        TorqueTareCoeffs = [0; -9.7616E-10; + 1.8563E-06; - 1.5695E-03; - 8.2200E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin019 w/28t pitch pulley
    elseif TestNo==25
        TorqueTareCoeffs = [1.6696E-12; - 5.4962E-09; + 5.8498E-06; - 2.8427E-03; - 5.9340E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin027 w/18t pitch pulley
    elseif TestNo<29
        TorqueTareCoeffs = [1.3947E-12; - 4.5325E-09; + 4.8100E-06; - 2.3151E-03; - 5.9495E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin020 w/23t pitch pulley
    elseif TestNo<32
        TorqueTareCoeffs = [-1.2169E-13; - 2.4978E-10; + 6.2385E-07; - 8.5693E-04; - 9.2603E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin029 w/38t pitch pulley
    elseif TestNo<33
        TorqueTareCoeffs = [-4.9119E-13; + 1.2347E-09; - 8.8961E-07; - 4.3711E-04; - 1.7436E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin032 w/28t pitch pulley
    elseif TestNo<35
        TorqueTareCoeffs = [2.2361E-12; - 5.8101E-09; + 5.2869E-06; - 2.9465E-03; - 1.0790E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin033 w/28t pitch pulley
        FC = 1/12; %fin mean chord for Fin "E" (ft)
    elseif TestNo<36
        TorqueTareCoeffs = [2.2361E-12; - 5.8101E-09; + 5.2869E-06; - 2.9465E-03; - 1.0790E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin033 w/28t pitch pulley
        FC = 1.5/12; %fin mean chord for Fin "D"(ft)
    elseif TestNo<42
        TorqueTareCoeffs = [2.2361E-12; - 5.8101E-09; + 5.2869E-06; - 2.9465E-03; - 1.0790E+00]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin033 w/28t pitch pulley
        FC = 2/12; %fin mean chord for Fin "C" (ft)
    else
        TorqueTareCoeffs = [-3.1454E-13; + 7.8328E-10; - 5.6519E-07; - 7.8943E-04; - 9.4018E-01]; %4th order polynomial, highest power first. Nm vs RPM. From Pterofin042 w/28t pitch pulley
    end
    
    % apply mean torque tare
    if ApplyTare
        Tare_FtLbf = 0.7376*(TorqueTareCoeffs(1).*MotorRPM.^4 + ...
            TorqueTareCoeffs(2).*MotorRPM.^3 + ...
            TorqueTareCoeffs(3).*MotorRPM.^2 + ...
            TorqueTareCoeffs(4).*MotorRPM.^1 + ...
            TorqueTareCoeffs(5)); % Torque tare, ft-lbf
        disp('Applying mean torque tare.')
    else
        Tare_FtLbf = 0;
        disp('Torque tare not applied.')
    end
    
    % Calculate power coefficient
    Torque_Nm_temp = Data{i}.Torque_Signal*10.027798; %torque calculated from voltage without mechanism tare applied (Nm)
    Torque_Tared_FtLbf = Torque_Nm_temp*0.7376 - Tare_FtLbf; %torque calculated from voltage with mechanism tare applied (ft-lbf)
    Torque_Nm_temp;
    Torque_Tared_Nm = Torque_Tared_FtLbf/0.7376; %torque calculated from voltage with mechanism tare applied (Nm)
    MotorPowerHP = MotorRPM.*Torque_Tared_FtLbf/5252; %Motor shaft power (hp)
    Cp = MotorPowerHP*550./(0.5*rho.*Vinf.^3*SweptArea);
    KQ = Torque_Tared_FtLbf./(rho.*(MotorRPM/60).^2*(Dtip_ft)^5);
    
    % Interpolate signals for phase average to unique integer motor angles
    [MotorPosition_unique,locsUniquePos,~] = unique(MotorPosition);
    vq = [round(MotorPosition(1)):1:round(MotorPosition(end))]';
    Torque_Nm_terp = pchip(MotorPosition_unique,Torque_Tared_Nm(locsUniquePos),vq);
    Tare_FtLbf_terp = pchip(MotorPosition_unique,Tare_FtLbf(locsUniquePos),vq);
    Cp_terp = pchip(MotorPosition_unique,Cp(locsUniquePos),vq);
    MotorRPM_terp = pchip(MotorPosition_unique,MotorRPM(locsUniquePos),vq);
    Kq_terp = pchip(MotorPosition_unique,KQ(locsUniquePos),vq);
    Vinf_terp = pchip(MotorPosition_unique,Vinf(locsUniquePos),vq);
    [~,locsFullFlappingPeriods_terp]= ismember(round(MotorPosition(locsFullFlappingPeriods)),vq);
    
    %locsFullFlappingPeriods_terp
    numBins=200;
    [Torque_Tared_Nm_PA,phase,numSamplesBin,phaseStdDevData_Q] = phaseAvg(Torque_Nm_terp,locsFullFlappingPeriods_terp,numBins);
    [Tare_FtLbf_PA,~,~,~] = phaseAvg(Tare_FtLbf_terp,locsFullFlappingPeriods_terp,numBins);
    [Cp_PA,~,~,phaseStdDevData_Cp] = phaseAvg(Cp_terp,locsFullFlappingPeriods_terp,numBins);
    [MotorRPM_PA,~,~,phaseStdDevData_RPM] = phaseAvg(MotorRPM_terp,locsFullFlappingPeriods_terp,numBins);
    [KQ_PA,~,~,phaseStdDevData_Kq] = phaseAvg(Kq_terp,locsFullFlappingPeriods_terp,numBins);
    [Vinf_PA,~,~,~] = phaseAvg(Vinf_terp,locsFullFlappingPeriods_terp,numBins);
    Torque_Tared_FtLbf_PA = Torque_Tared_Nm_PA*0.7376;
    
    %Period is based off two full rotations of the motor
    motorPeriod=mean(MotorRPM_PA)*2*pi/60;
    phaseTime=phase/(2*pi)*(2*motorPeriod);
    phaseAngle=phase/(2*pi)*360;
    Position_PA = phaseAngle*2; %phase averaged motor angle in degrees
    
    % Calculate Reduced Frequency
    FinFreq = MotorRPM_PA/60*pi; % rad/s, includes factor of 1/2 to account for 2:1 motor:fin speed ratio
    k_PA = FC*FinFreq./Vinf_PA; % reduced frequency of fin flapping and pitching motion
    
    % Calculate phase averaged motor power
    MotorPowerHP_PA = Cp_PA.*(0.5*rho.*Vinf_PA.^3*SweptArea)/550; %Motor shaft power (hp)
    
    % Calculate static chord-based Reynolds number (does not use blade relative
    % velocity)
    ReC_PA = FC*Vinf_PA./Viscosity;
    
    % Build a table of data for export to CSV
    
    Position_PA = Position_PA';
    k_PA = k_PA';
    Tare_FtLbf_PA = Tare_FtLbf_PA';
    Torque_Tared_Nm_PA = Torque_Tared_Nm_PA';
    Torque_Tared_FtLbf_PA = Torque_Tared_FtLbf_PA';
    MotorRPM_PA = MotorRPM_PA';
    KQ_PA = KQ_PA';
    MotorPowerHP_PA = MotorPowerHP_PA';
    Cp_PA = Cp_PA';
    ReC_PA = ReC_PA';
    T = table(Position_PA,k_PA,Tare_FtLbf_PA,Torque_Tared_Nm_PA,...;
        Torque_Tared_FtLbf_PA,MotorRPM_PA,KQ_PA,MotorPowerHP_PA,...;
        Cp_PA,ReC_PA);
    
    if SaveData
        if iscell(PathName)
            str = [PathName{i} FileName{i}(1:end-4) '_PHASE_AVERAGED.csv'];
        else
            str = [PathName FileName{i}(1:end-4) '_PHASE_AVERAGED.csv'];
        end
        
        writetable(T,str)
    end
    
    %Compare phase average signal with time signal. Using the first complete
    %flapping cycle (indices of 1 and 2)
    timeSignalSample=Time(locsFullFlappingPeriods(1):locsFullFlappingPeriods(2));
    timeSignalSampleShifted=timeSignalSample-timeSignalSample(1);
    torqueSignalSample=Torque_Tared_FtLbf(locsFullFlappingPeriods(1):locsFullFlappingPeriods(2))/0.7376;
    CpSignalSample=Cp(locsFullFlappingPeriods(1):locsFullFlappingPeriods(2));
    RPMSignalSample=MotorRPM(locsFullFlappingPeriods(1):locsFullFlappingPeriods(2));
    VinfSignalSample=Vinf(locsFullFlappingPeriods(1):locsFullFlappingPeriods(2));
    signalSamplePeriod=timeSignalSampleShifted(end);
    signalSampleDegrees=timeSignalSampleShifted/signalSamplePeriod*360;
    
    motorPeriod=1/(mean(MotorRPM(locsFullFlappingPeriods(1):locsFullFlappingPeriods(2)))/60);
    phaseND=phase/(2*pi);
    
    torqueMean=mean(Torque_Tared_FtLbf/0.7376);
    CpMean=mean(Cp);
    RPMMean=mean(MotorRPM);
    KqMean=mean(KQ);
    
    if MakePlots == 1
        
        figure;subplot(4,1,1);hold on
        plot(Position_PA,Torque_Tared_Nm_PA,'r')
        plot([Position_PA(1),Position_PA(end)],[torqueMean,torqueMean],'g')
        plot(Position_PA,Torque_Tared_Nm_PA+2*phaseStdDevData_Q','--k')
        plot(Position_PA,Torque_Tared_Nm_PA-2*phaseStdDevData_Q','--k')
        hold off
        xlabel('Motor Position (deg.)') %720 degrees (2 motor revolutions) = 1 fin cycle
        ylabel('Motor Torque, Q (Nm)')
        grid on
        
        subplot(4,1,2);hold on
        plot(Position_PA,MotorRPM_PA,'r')
        plot([Position_PA(1),Position_PA(end)],[RPMMean,RPMMean],'g')
        plot(Position_PA,MotorRPM_PA+2*phaseStdDevData_RPM','--k')
        plot(Position_PA,MotorRPM_PA-2*phaseStdDevData_RPM','--k')
        hold off
        xlabel('Motor Position (deg.)') %720 degrees (2 motor revolutions) = 1 fin cycle
        ylabel('Motor Speed (RPM)')
        grid on
        
        subplot(4,1,3);hold on
        plot(Position_PA,KQ_PA,'r')
        plot([Position_PA(1),Position_PA(end)],[KqMean,KqMean],'g')
        plot(Position_PA,KQ_PA+2*phaseStdDevData_Kq','--k')
        plot(Position_PA,KQ_PA-2*phaseStdDevData_Kq','--k')
        hold off
        xlabel('Motor Position (deg.)') %720 degrees (2 motor revolutions) = 1 fin cycle
        ylabel('Motor Torque Coef., {K_Q}')
        grid on
        
        subplot(4,1,4);hold on
        plot(Position_PA,Cp_PA,'r')
        plot([Position_PA(1),Position_PA(end)],[CpMean,CpMean],'g')
        plot(Position_PA,Cp_PA+2*phaseStdDevData_Cp','--k')
        plot(Position_PA,Cp_PA-2*phaseStdDevData_Cp','--k')
        hold off
        legend('Phase Average','Mean','Phase Average +/- 2{\sigma}','Location','best')
        xlabel('Motor Position (deg.)') %720 degrees (2 motor revolutions) = 1 fin cycle
        ylabel('Motor Power Coef., {C_P}')
        grid on
        
        titleString=['Phase Statistics: ' FileName{i}(1:end-4)];
        sgtitle(titleString)
        set(gcf,'Position',[50,50,600,800])
        if SavePlots
            str = [SavePath FileName{i}(1:end-4) '_phase_averaged_values.fig'];
            savefig(gcf,str)
            str = [SavePath FileName{i}(1:end-4) '_phase_averaged_values.png'];
            exportgraphics(gcf,str)
        end
    end
end