function [Data, TimeVector, Setup]=read_a_csv(loadfile)
%
%function [Data, TimeVector, Setup]=read_a_csv(loadfile)
% This function reads in the National Instruments tdms file format.
%
% use :  [Data, TimeVector, Setup]=read_a_csv(loadfile)
%
%	Inputs:
%		loadfile    - name the csv file to be read STRING
%                   - POPS a gui if not included
%
%	Outputs:
%		Data - Time history data from file - STRUCTURE of DOUBLE
%		TimeVector - Time Vector - DOUBLE
%       Setup - Setup information from the tdms file - STRUCTURE
%           .Data_Path - Data File - STRING
%           .TimeStamp - Time Stamp - STRING
%           .Fs - Sample Rate, Hz - DOUBLE
%           .npnts - Number of Data Points - INTEGER
%           .ChanNames - Channel Names - STRUCTURE OF STRINGS
%
% Who               When        Why
% D.W.DeVilbiss     Aug 2012    New and Necessary
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if(nargin==0)
    [FileName,PathName] = uigetfile('*.csv; *.xls','Select a *.csv or *.xls data file');
	loadfile=[PathName, FileName];
end
[pathstr, namestr, extstr] = fileparts(loadfile);
if(strcmp(extstr,'.csv'))
    %
    imp_data=importdata(loadfile,',');
    if(~isfield(imp_data, 'colheaders'))
        nvar=size(imp_data.data,2);
        [token, remain] = strtok(imp_data.textdata{1},',');
        imp_data.colheaders{1}=char(token);
        for ivar=2:nvar
            [token, remain] = strtok(remain,',');
            imp_data.colheaders{ivar}=char(token);
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},'.','');
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},' ','');
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},'(','_');
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},'/','_over_');
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},'\','_over_');
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},'*','_times_');
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},')','');
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},'^','');
            imp_data.colheaders{ivar}=strrep(imp_data.colheaders{ivar},'%','perc');
        end
    end
    %
    Setup.Data_Path=loadfile;
    file_info=dir(loadfile);
    Setup.TimeStamp = file_info.date;
    Setup.ChanNames = imp_data.colheaders;
    Setup.npnts=size(imp_data.data,1);
    for ivar=1:length(imp_data.colheaders)
        evalstr=['Data.',imp_data.colheaders{ivar},'=imp_data.data(:,ivar);'];
        eval(evalstr);
    end
    %
    if(isfield(Data,'Time'))
        TimeVector=Data.Time;
        Setup.Fs=1/(TimeVector(2)-TimeVector(1));
    else
        TimeVector=NaN(1,length(imp_data.data));
        Setup.Fs=NaN;
    end
    %
else
    [status,sheets] = xlsfinfo(loadfile);
    [data,colheaders,raw]=xlsread(loadfile,sheets{1});
    if(isempty(data))
        nsheets=length(sheets);
        evalstr=['[data,colheaders,raw]=xlsread(loadfile,''',sheets{nsheets},''');'];
        eval(evalstr);
    end
        
    %
    % Get rid of special characters in the column headers
    %
    for ivar=1:length(colheaders)
        colheaders{ivar}=strrep(colheaders{ivar},' ','');
        colheaders{ivar}=strrep(colheaders{ivar},'-','_');
        colheaders{ivar}=strrep(colheaders{ivar},'(','_');
        colheaders{ivar}=strrep(colheaders{ivar},')','');
        colheaders{ivar}=strrep(colheaders{ivar},'/','p');
        colheaders{ivar}=strrep(colheaders{ivar},'^','');
        colheaders{ivar}=strrep(colheaders{ivar},'%','perc');
    end
    Setup.Data_Path=loadfile;
    file_info=dir(loadfile);
    Setup.TimeStamp = file_info.date;
    Setup.ChanNames = colheaders;
    Setup.npnts=size(data,1);
    for ivar=1:length(colheaders)
       evalstr=['Data.',colheaders{ivar},'=data(:,ivar);'];
       eval(evalstr);
    end
    %
    if(isfield(Data,'Time'))
        TimeVector=Data.Time;
        Setup.Fs=1/(TimeVector(2)-TimeVector(1));
    else
        TimeVector=NaN(1,length(data));
        Setup.Fs=NaN;
    end
    %
end
%
return
end